@@ -1394,6 +1394,7 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
   gint outsize;
   gboolean rawvideo;
   GstFlowReturn stream_last_flow;
+  gint64 pts;
 
   /* open file if we didn't so already */
   if (!demux->opened)
@@ -1421,7 +1422,21 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
 
   /* do timestamps, we do this first so that we can know when we
    * stepped over the segment stop position. */
-  timestamp = gst_ffmpeg_time_ff_to_gst (pkt.pts, avstream->time_base);
+  pts = pkt.pts;
+  if (G_UNLIKELY (pts < 0)) {
+    /* some streams have pts such this:
+     * 0
+     * -2
+     * -1
+     * 1
+     *
+     * we reset pts to 0 since for us timestamp are unsigned
+     */
+    GST_WARNING_OBJECT (demux,
+        "negative pts detected: %" G_GINT64_FORMAT " resetting to 0", pts);
+    pts = 0;
+  }
+  timestamp = gst_ffmpeg_time_ff_to_gst (pts, avstream->time_base);
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
     stream->last_ts = timestamp;
   }
@@ -1473,17 +1488,10 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
         ((GstFFMpegDemuxClass *) (G_OBJECT_GET_CLASS (demux)))->in_plugin->name;
     GstMapInfo map;
 
-    if (strcmp (plugin_name, "gif") == 0) {
-      src.data[0] = pkt.data;
-      src.data[1] = NULL;
-      src.data[2] = NULL;
-      src.linesize[0] = avstream->codec->width * 3;
-    } else {
-      GST_WARNING ("Unknown demuxer %s, no idea what to do", plugin_name);
-      gst_ffmpeg_avpicture_fill (&src, pkt.data,
-          avstream->codec->pix_fmt, avstream->codec->width,
-          avstream->codec->height);
-    }
+    GST_WARNING ("Unknown demuxer %s, no idea what to do", plugin_name);
+    gst_ffmpeg_avpicture_fill (&src, pkt.data,
+        avstream->codec->pix_fmt, avstream->codec->width,
+        avstream->codec->height);
 
     gst_buffer_map (outbuf, &map, GST_MAP_WRITE);
     gst_ffmpeg_avpicture_fill (&dst, map.data,
@@ -2069,7 +2077,10 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
         !strcmp (in_plugin->name, "yuv4mpegpipe") ||
         !strcmp (in_plugin->name, "pva") ||
         !strcmp (in_plugin->name, "mpc") ||
-        !strcmp (in_plugin->name, "mpc8") || !strcmp (in_plugin->name, "ivf"))
+        !strcmp (in_plugin->name, "mpc8") ||
+        !strcmp (in_plugin->name, "ivf") ||
+        !strcmp (in_plugin->name, "brstm") ||
+        !strcmp (in_plugin->name, "bfstm") || !strcmp (in_plugin->name, "gif"))
       rank = GST_RANK_MARGINAL;
     else {
       GST_DEBUG ("ignoring %s", in_plugin->name);
