@@ -205,6 +205,9 @@ gst_ffmpegdata_close (AVIOContext * h)
 {
   GstProtocolInfo *info;
 
+  if (h == NULL)
+    return 0;
+
   info = (GstProtocolInfo *) h->opaque;
   if (info == NULL)
     return 0;
@@ -241,6 +244,7 @@ gst_ffmpegdata_open (GstPad * pad, int flags, AVIOContext ** context)
   /* we don't support R/W together */
   if ((flags & AVIO_FLAG_WRITE) && (flags & AVIO_FLAG_READ)) {
     GST_WARNING ("Only read-only or write-only are supported");
+    g_free (info);
     return -EINVAL;
   }
 
@@ -259,12 +263,19 @@ gst_ffmpegdata_open (GstPad * pad, int flags, AVIOContext ** context)
   buffer = av_malloc (buffer_size);
   if (buffer == NULL) {
     GST_WARNING ("Failed to allocate buffer");
+    g_free (info);
     return -ENOMEM;
   }
 
   *context =
       avio_alloc_context (buffer, buffer_size, flags, (void *) info,
       gst_ffmpegdata_read, gst_ffmpegdata_write, gst_ffmpegdata_seek);
+  if (*context == NULL) {
+    GST_WARNING ("Failed to allocate memory");
+    g_free (info);
+    av_free (buffer);
+    return -ENOMEM;
+  }
   (*context)->seekable = AVIO_SEEKABLE_NORMAL;
   if (!(flags & AVIO_FLAG_WRITE)) {
     (*context)->buf_ptr = (*context)->buf_end;
@@ -318,6 +329,9 @@ gst_ffmpeg_pipe_close (AVIOContext * h)
 {
   GST_LOG ("Closing pipe");
 
+  if (h == NULL)
+    return 0;
+
   h->opaque = NULL;
   av_freep (&h->buffer);
   av_free (h);
@@ -343,6 +357,11 @@ gst_ffmpeg_pipe_open (GstFFMpegPipe * ffpipe, int flags, AVIOContext ** context)
   *context =
       avio_alloc_context (buffer, buffer_size, 0, (void *) ffpipe,
       gst_ffmpeg_pipe_read, NULL, NULL);
+  if (*context == NULL) {
+    GST_WARNING ("Failed to allocate memory");
+    av_free (buffer);
+    return -ENOMEM;
+  }
   (*context)->seekable = 0;
   (*context)->buf_ptr = (*context)->buf_end;
 
