@@ -24,6 +24,7 @@
 #include "libavutil/avassert.h"
 #include "libavutil/avutil.h"
 #include "libavutil/colorspace.h"
+#include "libavutil/intreadwrite.h"
 #include "libavutil/mem.h"
 #include "libavutil/pixdesc.h"
 #include "drawutils.h"
@@ -52,11 +53,23 @@ int ff_fill_rgba_map(uint8_t *rgba_map, enum AVPixelFormat pix_fmt)
     case AV_PIX_FMT_BGRA:
     case AV_PIX_FMT_BGR0:
     case AV_PIX_FMT_BGR24: rgba_map[BLUE ] = 0; rgba_map[GREEN] = 1; rgba_map[RED  ] = 2; rgba_map[ALPHA] = 3; break;
-    case AV_PIX_FMT_GBRP9:
-    case AV_PIX_FMT_GBRP10:
-    case AV_PIX_FMT_GBRP12:
-    case AV_PIX_FMT_GBRP14:
+    case AV_PIX_FMT_GBRP9LE:
+    case AV_PIX_FMT_GBRP9BE:
+    case AV_PIX_FMT_GBRP10LE:
+    case AV_PIX_FMT_GBRP10BE:
+    case AV_PIX_FMT_GBRP12LE:
+    case AV_PIX_FMT_GBRP12BE:
+    case AV_PIX_FMT_GBRP14LE:
+    case AV_PIX_FMT_GBRP14BE:
+    case AV_PIX_FMT_GBRP16LE:
+    case AV_PIX_FMT_GBRP16BE:
     case AV_PIX_FMT_GBRAP:
+    case AV_PIX_FMT_GBRAP10LE:
+    case AV_PIX_FMT_GBRAP10BE:
+    case AV_PIX_FMT_GBRAP12LE:
+    case AV_PIX_FMT_GBRAP12BE:
+    case AV_PIX_FMT_GBRAP16LE:
+    case AV_PIX_FMT_GBRAP16BE:
     case AV_PIX_FMT_GBRP:  rgba_map[GREEN] = 0; rgba_map[BLUE ] = 1; rgba_map[RED  ] = 2; rgba_map[ALPHA] = 3; break;
     default:                    /* unsupported */
         return AVERROR(EINVAL);
@@ -173,10 +186,14 @@ int ff_draw_init(FFDrawContext *draw, enum AVPixelFormat format, unsigned flags)
         return AVERROR(EINVAL);
     if (desc->flags & ~(AV_PIX_FMT_FLAG_PLANAR | AV_PIX_FMT_FLAG_RGB | AV_PIX_FMT_FLAG_PSEUDOPAL | AV_PIX_FMT_FLAG_ALPHA))
         return AVERROR(ENOSYS);
+    if (format == AV_PIX_FMT_P010LE || format == AV_PIX_FMT_P010BE)
+        return AVERROR(ENOSYS);
     for (i = 0; i < desc->nb_components; i++) {
         c = &desc->comp[i];
-        /* for now, only 8-bits formats */
-        if (c->depth != 8)
+        /* for now, only 8-16 bits formats */
+        if (c->depth < 8 || c->depth > 16)
+            return AVERROR(ENOSYS);
+        if (desc->flags & AV_PIX_FMT_FLAG_BE)
             return AVERROR(ENOSYS);
         if (c->plane >= MAX_PLANES)
             return AVERROR(ENOSYS);
@@ -184,21 +201,23 @@ int ff_draw_init(FFDrawContext *draw, enum AVPixelFormat format, unsigned flags)
         if (pixelstep[c->plane] != 0 &&
             pixelstep[c->plane] != c->step)
             return AVERROR(ENOSYS);
+        if (pixelstep[c->plane] == 6 &&
+            c->depth == 16)
+            return AVERROR(ENOSYS);
         pixelstep[c->plane] = c->step;
         if (pixelstep[c->plane] >= 8)
             return AVERROR(ENOSYS);
         nb_planes = FFMAX(nb_planes, c->plane + 1);
     }
-    if ((desc->log2_chroma_w || desc->log2_chroma_h) && nb_planes < 3)
-        return AVERROR(ENOSYS); /* exclude NV12 and NV21 */
     memset(draw, 0, sizeof(*draw));
     draw->desc      = desc;
     draw->format    = format;
     draw->nb_planes = nb_planes;
+    draw->flags     = flags;
     memcpy(draw->pixelstep, pixelstep, sizeof(draw->pixelstep));
     draw->hsub[1] = draw->hsub[2] = draw->hsub_max = desc->log2_chroma_w;
     draw->vsub[1] = draw->vsub[2] = draw->vsub_max = desc->log2_chroma_h;
-    for (i = 0; i < ((desc->nb_components - 1) | 1); i++)
+    for (i = 0; i < (desc->nb_components - !!(desc->flags & AV_PIX_FMT_FLAG_ALPHA && !(flags & FF_DRAW_PROCESS_ALPHA))); i++)
         draw->comp_mask[desc->comp[i].plane] |=
             1 << desc->comp[i].offset;
     return 0;
@@ -214,21 +233,45 @@ void ff_draw_color(FFDrawContext *draw, FFDrawColor *color, const uint8_t rgba[4
     if ((draw->desc->flags & AV_PIX_FMT_FLAG_RGB) &&
         ff_fill_rgba_map(rgba_map, draw->format) >= 0) {
         if (draw->nb_planes == 1) {
-            for (i = 0; i < 4; i++)
+            for (i = 0; i < 4; i++) {
                 color->comp[0].u8[rgba_map[i]] = rgba[i];
+                if (draw->desc->comp[rgba_map[i]].depth > 8) {
+                    color->comp[0].u16[rgba_map[i]] = color->comp[0].u8[rgba_map[i]] << 8;
+                }
+            }
         } else {
-            for (i = 0; i < 4; i++)
+            for (i = 0; i < 4; i++) {
                 color->comp[rgba_map[i]].u8[0] = rgba[i];
+                if (draw->desc->comp[rgba_map[i]].depth > 8)
+                    color->comp[rgba_map[i]].u16[0] = color->comp[rgba_map[i]].u8[0] << (draw->desc->comp[rgba_map[i]].depth - 8);
+            }
         }
-    } else if (draw->nb_planes == 3 || draw->nb_planes == 4) {
+    } else if (draw->nb_planes >= 2) {
         /* assume YUV */
-        color->comp[0].u8[0] = RGB_TO_Y_CCIR(rgba[0], rgba[1], rgba[2]);
-        color->comp[1].u8[0] = RGB_TO_U_CCIR(rgba[0], rgba[1], rgba[2], 0);
-        color->comp[2].u8[0] = RGB_TO_V_CCIR(rgba[0], rgba[1], rgba[2], 0);
+        const AVPixFmtDescriptor *desc = draw->desc;
+        color->comp[desc->comp[0].plane].u8[desc->comp[0].offset] = RGB_TO_Y_CCIR(rgba[0], rgba[1], rgba[2]);
+        color->comp[desc->comp[1].plane].u8[desc->comp[1].offset] = RGB_TO_U_CCIR(rgba[0], rgba[1], rgba[2], 0);
+        color->comp[desc->comp[2].plane].u8[desc->comp[2].offset] = RGB_TO_V_CCIR(rgba[0], rgba[1], rgba[2], 0);
         color->comp[3].u8[0] = rgba[3];
-    } else if (draw->format == AV_PIX_FMT_GRAY8 || draw->format == AV_PIX_FMT_GRAY8A) {
+#define EXPAND(compn) \
+        if (desc->comp[compn].depth > 8) \
+            color->comp[desc->comp[compn].plane].u16[desc->comp[compn].offset] = \
+            color->comp[desc->comp[compn].plane].u8[desc->comp[compn].offset] << \
+                (draw->desc->comp[compn].depth + draw->desc->comp[compn].shift - 8)
+        EXPAND(3);
+        EXPAND(2);
+        EXPAND(1);
+        EXPAND(0);
+    } else if (draw->format == AV_PIX_FMT_GRAY8 || draw->format == AV_PIX_FMT_GRAY8A ||
+               draw->format == AV_PIX_FMT_GRAY16LE || draw->format == AV_PIX_FMT_YA16LE ||
+               draw->format == AV_PIX_FMT_GRAY9LE  ||
+               draw->format == AV_PIX_FMT_GRAY10LE ||
+               draw->format == AV_PIX_FMT_GRAY12LE) {
+        const AVPixFmtDescriptor *desc = draw->desc;
         color->comp[0].u8[0] = RGB_TO_Y_CCIR(rgba[0], rgba[1], rgba[2]);
+        EXPAND(0);
         color->comp[1].u8[0] = rgba[3];
+        EXPAND(1);
     } else {
         av_log(NULL, AV_LOG_WARNING,
                "Color conversion not implemented for %s\n", draw->desc->name);
@@ -272,6 +315,7 @@ void ff_fill_rectangle(FFDrawContext *draw, FFDrawColor *color,
 {
     int plane, x, y, wp, hp;
     uint8_t *p0, *p;
+    FFDrawColor color_tmp = *color;
 
     for (plane = 0; plane < draw->nb_planes; plane++) {
         p0 = pointer_at(draw, dst, dst_linesize, plane, dst_x, dst_y);
@@ -280,9 +324,15 @@ void ff_fill_rectangle(FFDrawContext *draw, FFDrawColor *color,
         if (!hp)
             return;
         p = p0;
+
+        if (HAVE_BIGENDIAN && draw->desc->comp[0].depth > 8) {
+            for (x = 0; 2*x < draw->pixelstep[plane]; x++)
+                color_tmp.comp[plane].u16[x] = av_bswap16(color_tmp.comp[plane].u16[x]);
+        }
+
         /* copy first line from color */
         for (x = 0; x < wp; x++) {
-            memcpy(p, color->comp[plane].u8, draw->pixelstep[plane]);
+            memcpy(p, color_tmp.comp[plane].u8, draw->pixelstep[plane]);
             p += draw->pixelstep[plane];
         }
         wp *= draw->pixelstep[plane];
@@ -361,6 +411,31 @@ static void blend_line(uint8_t *dst, unsigned src, unsigned alpha,
     }
 }
 
+static void blend_line16(uint8_t *dst, unsigned src, unsigned alpha,
+                         int dx, int w, unsigned hsub, int left, int right)
+{
+    unsigned asrc = alpha * src;
+    unsigned tau = 0x10001 - alpha;
+    int x;
+
+    if (left) {
+        unsigned suba = (left * alpha) >> hsub;
+        uint16_t value = AV_RL16(dst);
+        AV_WL16(dst, (value * (0x10001 - suba) + src * suba) >> 16);
+        dst += dx;
+    }
+    for (x = 0; x < w; x++) {
+        uint16_t value = AV_RL16(dst);
+        AV_WL16(dst, (value * tau + asrc) >> 16);
+        dst += dx;
+    }
+    if (right) {
+        unsigned suba = (right * alpha) >> hsub;
+        uint16_t value = AV_RL16(dst);
+        AV_WL16(dst, (value * (0x10001 - suba) + src * suba) >> 16);
+    }
+}
+
 void ff_blend_rectangle(FFDrawContext *draw, FFDrawColor *color,
                         uint8_t *dst[], int dst_linesize[],
                         int dst_w, int dst_h,
@@ -375,9 +450,15 @@ void ff_blend_rectangle(FFDrawContext *draw, FFDrawColor *color,
     clip_interval(dst_h, &y0, &h, NULL);
     if (w <= 0 || h <= 0 || !color->rgba[3])
         return;
-    /* 0x10203 * alpha + 2 is in the [ 2 ; 0x1010101 - 2 ] range */
-    alpha = 0x10203 * color->rgba[3] + 0x2;
-    nb_planes = (draw->nb_planes - 1) | 1; /* eliminate alpha */
+    if (draw->desc->comp[0].depth <= 8) {
+        /* 0x10203 * alpha + 2 is in the [ 2 ; 0x1010101 - 2 ] range */
+        alpha = 0x10203 * color->rgba[3] + 0x2;
+    } else {
+        /* 0x101 * alpha is in the [ 2 ; 0x1001] range */
+        alpha = 0x101 * color->rgba[3] + 0x2;
+    }
+    nb_planes = draw->nb_planes - !!(draw->desc->flags & AV_PIX_FMT_FLAG_ALPHA && !(draw->flags & FF_DRAW_PROCESS_ALPHA));
+    nb_planes += !nb_planes;
     for (plane = 0; plane < nb_planes; plane++) {
         nb_comp = draw->pixelstep[plane];
         p0 = pointer_at(draw, dst, dst_linesize, plane, x0, y0);
@@ -388,29 +469,77 @@ void ff_blend_rectangle(FFDrawContext *draw, FFDrawColor *color,
         subsampling_bounds(draw->hsub[plane], &x_sub, &w_sub, &left, &right);
         subsampling_bounds(draw->vsub[plane], &y_sub, &h_sub, &top, &bottom);
         for (comp = 0; comp < nb_comp; comp++) {
+            const int depth = draw->desc->comp[comp].depth;
+
             if (!component_used(draw, plane, comp))
                 continue;
             p = p0 + comp;
             if (top) {
-                blend_line(p, color->comp[plane].u8[comp], alpha >> 1,
-                           draw->pixelstep[plane], w_sub,
-                           draw->hsub[plane], left, right);
+                if (depth <= 8) {
+                    blend_line(p, color->comp[plane].u8[comp], alpha >> 1,
+                               draw->pixelstep[plane], w_sub,
+                               draw->hsub[plane], left, right);
+                } else {
+                    blend_line16(p, color->comp[plane].u16[comp], alpha >> 1,
+                                 draw->pixelstep[plane], w_sub,
+                                 draw->hsub[plane], left, right);
+                }
                 p += dst_linesize[plane];
             }
-            for (y = 0; y < h_sub; y++) {
-                blend_line(p, color->comp[plane].u8[comp], alpha,
-                           draw->pixelstep[plane], w_sub,
-                           draw->hsub[plane], left, right);
-                p += dst_linesize[plane];
+            if (depth <= 8) {
+                for (y = 0; y < h_sub; y++) {
+                    blend_line(p, color->comp[plane].u8[comp], alpha,
+                               draw->pixelstep[plane], w_sub,
+                               draw->hsub[plane], left, right);
+                    p += dst_linesize[plane];
+                }
+            } else {
+                for (y = 0; y < h_sub; y++) {
+                    blend_line16(p, color->comp[plane].u16[comp], alpha,
+                                 draw->pixelstep[plane], w_sub,
+                                 draw->hsub[plane], left, right);
+                    p += dst_linesize[plane];
+                }
+            }
+            if (bottom) {
+                if (depth <= 8) {
+                    blend_line(p, color->comp[plane].u8[comp], alpha >> 1,
+                               draw->pixelstep[plane], w_sub,
+                               draw->hsub[plane], left, right);
+                } else {
+                    blend_line16(p, color->comp[plane].u16[comp], alpha >> 1,
+                                 draw->pixelstep[plane], w_sub,
+                                 draw->hsub[plane], left, right);
+                }
             }
-            if (bottom)
-                blend_line(p, color->comp[plane].u8[comp], alpha >> 1,
-                           draw->pixelstep[plane], w_sub,
-                           draw->hsub[plane], left, right);
         }
     }
 }
 
+static void blend_pixel16(uint8_t *dst, unsigned src, unsigned alpha,
+                          const uint8_t *mask, int mask_linesize, int l2depth,
+                          unsigned w, unsigned h, unsigned shift, unsigned xm0)
+{
+    unsigned xm, x, y, t = 0;
+    unsigned xmshf = 3 - l2depth;
+    unsigned xmmod = 7 >> l2depth;
+    unsigned mbits = (1 << (1 << l2depth)) - 1;
+    unsigned mmult = 255 / mbits;
+    uint16_t value = AV_RL16(dst);
+
+    for (y = 0; y < h; y++) {
+        xm = xm0;
+        for (x = 0; x < w; x++) {
+            t += ((mask[xm >> xmshf] >> ((~xm & xmmod) << l2depth)) & mbits)
+                 * mmult;
+            xm++;
+        }
+        mask += mask_linesize;
+    }
+    alpha = (t >> shift) * alpha;
+    AV_WL16(dst, ((0x10001 - alpha) * value + alpha * src) >> 16);
+}
+
 static void blend_pixel(uint8_t *dst, unsigned src, unsigned alpha,
                         const uint8_t *mask, int mask_linesize, int l2depth,
                         unsigned w, unsigned h, unsigned shift, unsigned xm0)
@@ -434,6 +563,31 @@ static void blend_pixel(uint8_t *dst, unsigned src, unsigned alpha,
     *dst = ((0x1010101 - alpha) * *dst + alpha * src) >> 24;
 }
 
+static void blend_line_hv16(uint8_t *dst, int dst_delta,
+                            unsigned src, unsigned alpha,
+                            const uint8_t *mask, int mask_linesize, int l2depth, int w,
+                            unsigned hsub, unsigned vsub,
+                            int xm, int left, int right, int hband)
+{
+    int x;
+
+    if (left) {
+        blend_pixel16(dst, src, alpha, mask, mask_linesize, l2depth,
+                      left, hband, hsub + vsub, xm);
+        dst += dst_delta;
+        xm += left;
+    }
+    for (x = 0; x < w; x++) {
+        blend_pixel16(dst, src, alpha, mask, mask_linesize, l2depth,
+                      1 << hsub, hband, hsub + vsub, xm);
+        dst += dst_delta;
+        xm += 1 << hsub;
+    }
+    if (right)
+        blend_pixel16(dst, src, alpha, mask, mask_linesize, l2depth,
+                      right, hband, hsub + vsub, xm);
+}
+
 static void blend_line_hv(uint8_t *dst, int dst_delta,
                           unsigned src, unsigned alpha,
                           const uint8_t *mask, int mask_linesize, int l2depth, int w,
@@ -474,10 +628,15 @@ void ff_blend_mask(FFDrawContext *draw, FFDrawColor *color,
     mask += ym0 * mask_linesize;
     if (mask_w <= 0 || mask_h <= 0 || !color->rgba[3])
         return;
-    /* alpha is in the [ 0 ; 0x10203 ] range,
-       alpha * mask is in the [ 0 ; 0x1010101 - 4 ] range */
-    alpha = (0x10307 * color->rgba[3] + 0x3) >> 8;
-    nb_planes = (draw->nb_planes - 1) | 1; /* eliminate alpha */
+    if (draw->desc->comp[0].depth <= 8) {
+        /* alpha is in the [ 0 ; 0x10203 ] range,
+           alpha * mask is in the [ 0 ; 0x1010101 - 4 ] range */
+        alpha = (0x10307 * color->rgba[3] + 0x3) >> 8;
+    } else {
+        alpha = (0x101 * color->rgba[3] + 0x2) >> 8;
+    }
+    nb_planes = draw->nb_planes - !!(draw->desc->flags & AV_PIX_FMT_FLAG_ALPHA && !(draw->flags & FF_DRAW_PROCESS_ALPHA));
+    nb_planes += !nb_planes;
     for (plane = 0; plane < nb_planes; plane++) {
         nb_comp = draw->pixelstep[plane];
         p0 = pointer_at(draw, dst, dst_linesize, plane, x0, y0);
@@ -488,34 +647,65 @@ void ff_blend_mask(FFDrawContext *draw, FFDrawColor *color,
         subsampling_bounds(draw->hsub[plane], &x_sub, &w_sub, &left, &right);
         subsampling_bounds(draw->vsub[plane], &y_sub, &h_sub, &top, &bottom);
         for (comp = 0; comp < nb_comp; comp++) {
+            const int depth = draw->desc->comp[comp].depth;
+
             if (!component_used(draw, plane, comp))
                 continue;
             p = p0 + comp;
             m = mask;
             if (top) {
-                blend_line_hv(p, draw->pixelstep[plane],
-                              color->comp[plane].u8[comp], alpha,
-                              m, mask_linesize, l2depth, w_sub,
-                              draw->hsub[plane], draw->vsub[plane],
-                              xm0, left, right, top);
+                if (depth <= 8) {
+                    blend_line_hv(p, draw->pixelstep[plane],
+                                  color->comp[plane].u8[comp], alpha,
+                                  m, mask_linesize, l2depth, w_sub,
+                                  draw->hsub[plane], draw->vsub[plane],
+                                  xm0, left, right, top);
+                } else {
+                    blend_line_hv16(p, draw->pixelstep[plane],
+                                    color->comp[plane].u16[comp], alpha,
+                                    m, mask_linesize, l2depth, w_sub,
+                                    draw->hsub[plane], draw->vsub[plane],
+                                    xm0, left, right, top);
+                }
                 p += dst_linesize[plane];
                 m += top * mask_linesize;
             }
-            for (y = 0; y < h_sub; y++) {
-                blend_line_hv(p, draw->pixelstep[plane],
-                              color->comp[plane].u8[comp], alpha,
-                              m, mask_linesize, l2depth, w_sub,
-                              draw->hsub[plane], draw->vsub[plane],
-                              xm0, left, right, 1 << draw->vsub[plane]);
-                p += dst_linesize[plane];
-                m += mask_linesize << draw->vsub[plane];
+            if (depth <= 8) {
+                for (y = 0; y < h_sub; y++) {
+                    blend_line_hv(p, draw->pixelstep[plane],
+                                  color->comp[plane].u8[comp], alpha,
+                                  m, mask_linesize, l2depth, w_sub,
+                                  draw->hsub[plane], draw->vsub[plane],
+                                  xm0, left, right, 1 << draw->vsub[plane]);
+                    p += dst_linesize[plane];
+                    m += mask_linesize << draw->vsub[plane];
+                }
+            } else {
+                for (y = 0; y < h_sub; y++) {
+                    blend_line_hv16(p, draw->pixelstep[plane],
+                                    color->comp[plane].u16[comp], alpha,
+                                    m, mask_linesize, l2depth, w_sub,
+                                    draw->hsub[plane], draw->vsub[plane],
+                                    xm0, left, right, 1 << draw->vsub[plane]);
+                    p += dst_linesize[plane];
+                    m += mask_linesize << draw->vsub[plane];
+                }
+            }
+            if (bottom) {
+                if (depth <= 8) {
+                    blend_line_hv(p, draw->pixelstep[plane],
+                                  color->comp[plane].u8[comp], alpha,
+                                  m, mask_linesize, l2depth, w_sub,
+                                  draw->hsub[plane], draw->vsub[plane],
+                                  xm0, left, right, bottom);
+                } else {
+                    blend_line_hv16(p, draw->pixelstep[plane],
+                                    color->comp[plane].u16[comp], alpha,
+                                    m, mask_linesize, l2depth, w_sub,
+                                    draw->hsub[plane], draw->vsub[plane],
+                                    xm0, left, right, bottom);
+                }
             }
-            if (bottom)
-                blend_line_hv(p, draw->pixelstep[plane],
-                              color->comp[plane].u8[comp], alpha,
-                              m, mask_linesize, l2depth, w_sub,
-                              draw->hsub[plane], draw->vsub[plane],
-                              xm0, left, right, bottom);
         }
     }
 }
@@ -545,43 +735,3 @@ AVFilterFormats *ff_draw_supported_pixel_formats(unsigned flags)
             return NULL;
     return fmts;
 }
-
-#ifdef TEST
-
-#undef printf
-
-int main(void)
-{
-    enum AVPixelFormat f;
-    const AVPixFmtDescriptor *desc;
-    FFDrawContext draw;
-    FFDrawColor color;
-    int r, i;
-
-    for (f = 0; av_pix_fmt_desc_get(f); f++) {
-        desc = av_pix_fmt_desc_get(f);
-        if (!desc->name)
-            continue;
-        printf("Testing %s...%*s", desc->name,
-               (int)(16 - strlen(desc->name)), "");
-        r = ff_draw_init(&draw, f, 0);
-        if (r < 0) {
-            char buf[128];
-            av_strerror(r, buf, sizeof(buf));
-            printf("no: %s\n", buf);
-            continue;
-        }
-        ff_draw_color(&draw, &color, (uint8_t[]) { 1, 0, 0, 1 });
-        for (i = 0; i < sizeof(color); i++)
-            if (((uint8_t *)&color)[i] != 128)
-                break;
-        if (i == sizeof(color)) {
-            printf("fallback color\n");
-            continue;
-        }
-        printf("ok\n");
-    }
-    return 0;
-}
-
-#endif
